from component.messagebus import publisher
from module.snake_module_name.component.repo.snake_entity_name_repo import (
    snake_entity_name_repo
)
from module.snake_module_name.entity.snake_entity_name.model import (
    PascalEntityNameModel
)

snake_entity_name_model: PascalEntityNameModel = PascalEntityNameModel(
    snake_entity_name_repo, publisher
)
